# -*- coding: utf8 -*-
# Text Formatter
# by Olivier R.
# License: MPL 2

import unohelper
import uno
import sys
import os
import traceback
import time

if sys.version_info.major == 3:
    import imp

import tf_strings
import tf_options


from com.sun.star.task import XJobExecutor
from com.sun.star.awt import XActionListener


#XRay - API explorer
from com.sun.star.uno import RuntimeException as _rtex
def xray (myObject):
    try:
        sm = uno.getComponentContext().ServiceManager
        mspf = sm.createInstanceWithContext("com.sun.star.script.provider.MasterScriptProviderFactory", uno.getComponentContext())
        scriptPro = mspf.createScriptProvider("")
        xScript = scriptPro.getScript("vnd.sun.star.script:XrayTool._Main.Xray?language=Basic&location=application")
        xScript.invoke((myObject,), (), ())
        return
    except:
        raise _rtex("\nBasic library Xray is not installed", uno.getComponentContext())

# from com.sun.star.awt.VclWindowPeerAttribute import OK, OK_CANCEL, YES_NO, YES_NO_CANCEL, RETRY_CANCEL, DEF_OK, DEF_CANCEL, DEF_RETRY, DEF_YES, DEF_NO
# def MessageBox (xParentWin, sMsg, sTitle, sBoxType="messbox", MsgButtons=OK):
#     if sBoxType not in ("messbox", "infobox", "errorbox", "warningbox", "querybox"):
#         sBoxType = "messbox"

#     # window properties
#     aDescriptor = uno.createUnoStruct("com.sun.star.awt.WindowDescriptor")
#     aDescriptor.Type = uno.Enum("com.sun.star.awt.WindowClass", "MODALTOP")
#     aDescriptor.WindowServiceName = sBoxType
#     aDescriptor.ParentIndex = -1
#     aDescriptor.Parent = xParentWin
#     #aDescriptor.Bounds = Rectangle()
#     aDescriptor.WindowAttributes = MsgButtons

#     xTK = xParentWin.getToolkit()
#     xMsgBox = xTK.createWindow(aDescriptor)
#     xMsgBox.setMessageText(sMsg)
#     xMsgBox.setCaptionText(sTitle)

#     return xMsgBox.execute()


class TextFormatter (unohelper.Base, XActionListener, XJobExecutor):
    def __init__ (self, ctx):
        self.ctx = ctx
        self.xSvMgr = self.ctx.ServiceManager
        self.xContainer = None
        self.xDialog = None
        # In this extension, fr is default language.
        # It is assumed that those who type in French understand French and may not understand English.
        xSettings = getConfigSetting("/org.openoffice.Setup/L10N", False)
        sLocale = xSettings.getByName("ooLocale")  # Note: look at ooSetupSystemLocale value ?
        if sLocale[0:2] in tf_strings.dStrings:
            self.dUI = tf_strings.dStrings[sLocale[0:2]]
        else:
            self.dUI = tf_strings.dStrings["fr"]
        
    def trigger (self, args):
        try:
            xTF = TextFormatter(self.ctx)
            xTF.run()
        except:
            traceback.print_exc()

    def _addWidget (self, name, wtype, x, y, w, h, **kwargs):
        xWidget = self.xDialog.createInstance('com.sun.star.awt.UnoControl%sModel' % wtype)
        xWidget.Name = name
        xWidget.PositionX = x
        xWidget.PositionY = y
        xWidget.Width = w
        xWidget.Height = h
        for k, w in kwargs.items():
            setattr(xWidget, k, w)
        self.xDialog.insertByName(name, xWidget)
        return xWidget

    def run (self):
        ## dialog
        self.xDialog = self.xSvMgr.createInstanceWithContext('com.sun.star.awt.UnoControlDialogModel', self.ctx)
        self.xDialog.Width = 320
        self.xDialog.Title = self.dUI.get('title', "#title#")

        xFD1 = uno.createUnoStruct("com.sun.star.awt.FontDescriptor")
        xFD1.Height = 12
        xFD1.Name = "Verdana"
        
        xFD2 = uno.createUnoStruct("com.sun.star.awt.FontDescriptor")
        xFD2.Height = 10
        xFD2.Weight = uno.getConstantByName("com.sun.star.awt.FontWeight.BOLD")
        xFD2.Name = "Verdana"
        
        xFDsmall = uno.createUnoStruct("com.sun.star.awt.FontDescriptor")
        xFDsmall.Height = 7
        xFDsmall.Name = "Verdana"

        ## widgets
        nGroupBoxWith = (self.xDialog.Width - 15) // 2
        nWidth = nGroupBoxWith-10
        nWidthHalf = (nWidth // 2) - 10
        nHeight = 10
        nColor = 0xAA2200
        
        # close or apply
        self.bClose = False

        # Python version
        self._addWidget('lblpython', 'FixedText', self.xDialog.Width-75, 0, 70, 10, Label = "Python {0[0]}.{0[1]}.{0[2]}".format(sys.version_info), Align = 2, TextColor = 0xDDDDDD, FontRelief = 1)

        # group box // surnumerary spaces
        nX1 = 10; nY1 = 5
        nPosRes = nGroupBoxWith - 22
        gbm1 = self._addWidget('groupbox1', 'GroupBox', nX1-5, nY1, nGroupBoxWith, 90, Label = "  " * len(self.dUI.get('ssp', "#error#")), FontDescriptor = xFD1)
        self.ssp = self._addWidget('ssp', 'CheckBox', nX1, nY1+2, nWidth, nHeight, Label = self.dUI.get('ssp', "#error#"), FontDescriptor = xFD1, FontRelief = 1, TextColor = nColor, State = True)
        self.ssp1 = self._addWidget('ssp1', 'CheckBox', nX1, nY1+15, nWidth, nHeight, Label = self.dUI.get('ssp1', "#error#"), State = True)
        self.ssp2 = self._addWidget('ssp2', 'CheckBox', nX1, nY1+25, nWidth, nHeight, Label = self.dUI.get('ssp2', "#error#"), State = True)
        self.ssp3 = self._addWidget('ssp3', 'CheckBox', nX1, nY1+35, nWidth, nHeight, Label = self.dUI.get('ssp3', "#error#"), State = True)
        self.ssp4 = self._addWidget('ssp4', 'CheckBox', nX1, nY1+45, nWidth, nHeight, Label = self.dUI.get('ssp4', "#error#"), State = True)
        self.ssp5 = self._addWidget('ssp5', 'CheckBox', nX1, nY1+55, nWidth, nHeight, Label = self.dUI.get('ssp5', "#error#"), State = True)
        self.ssp6 = self._addWidget('ssp6', 'CheckBox', nX1, nY1+65, nWidth, nHeight, Label = self.dUI.get('ssp6', "#error#"), State = True)
        self.ssp7 = self._addWidget('ssp7', 'CheckBox', nX1, nY1+75, nWidth, nHeight, Label = self.dUI.get('ssp7', "#error#"), State = True)
        self.ssp1_res = self._addWidget('ssp1_res', 'FixedText', nPosRes, nY1+15, 20, nHeight, Label = "", Align = 2)
        self.ssp2_res = self._addWidget('ssp2_res', 'FixedText', nPosRes, nY1+25, 20, nHeight, Label = "", Align = 2)
        self.ssp3_res = self._addWidget('ssp3_res', 'FixedText', nPosRes, nY1+35, 20, nHeight, Label = "", Align = 2)
        self.ssp4_res = self._addWidget('ssp4_res', 'FixedText', nPosRes, nY1+45, 20, nHeight, Label = "", Align = 2)
        self.ssp5_res = self._addWidget('ssp5_res', 'FixedText', nPosRes, nY1+55, 20, nHeight, Label = "", Align = 2)
        self.ssp6_res = self._addWidget('ssp6_res', 'FixedText', nPosRes, nY1+65, 20, nHeight, Label = "", Align = 2)
        self.ssp7_res = self._addWidget('ssp7_res', 'FixedText', nPosRes, nY1+75, 20, nHeight, Label = "", Align = 2)

        # group box // missing spaces
        nX2 = 10; nY2 = 100
        gbm2 = self._addWidget('groupbox2', 'GroupBox', nX2-5, nY2, nGroupBoxWith, 40, Label = "  " * len(self.dUI.get('space', "#error#")), FontDescriptor = xFD1)
        self.space = self._addWidget('space', 'CheckBox', nX2, nY2+2, nWidth, nHeight, Label = self.dUI.get('space', "#error#"), FontDescriptor = xFD1, FontRelief = 1, TextColor = nColor, State = True)
        self.space1 = self._addWidget('space1', 'CheckBox', nX2, nY2+15, nWidth, nHeight, Label = self.dUI.get('space1', "#error#"), State = True)
        self.space2 = self._addWidget('space2', 'CheckBox', nX2, nY2+25, nWidth, nHeight, Label = self.dUI.get('space2', "#error#"), State = True)
        self.space1_res = self._addWidget('space1_res', 'FixedText', nPosRes, nY2+15, 20, nHeight, Label = "", Align = 2)
        self.space2_res = self._addWidget('space2_res', 'FixedText', nPosRes, nY2+25, 20, nHeight, Label = "", Align = 2)
        
        # group box // non-breaking spaces
        nX3 = 10; nY3 = 145
        gbm3 = self._addWidget('groupbox3', 'GroupBox', nX3-5, nY3, nGroupBoxWith, 60, Label = "  " * len(self.dUI.get('nbsp', "#error#")), FontDescriptor = xFD1)
        self.nbsp = self._addWidget('nbsp', 'CheckBox', nX3, nY3+2, nWidth, nHeight, Label = self.dUI.get('nbsp', "#error#"), FontDescriptor = xFD1, FontRelief = 1, TextColor = nColor, State = True)
        self.nbsp1 = self._addWidget('nbsp1', 'CheckBox', nX3, nY3+15, 85, nHeight, Label = self.dUI.get('nbsp1', "#error#"), State = True)
        self.nbsp2 = self._addWidget('nbsp2', 'CheckBox', nX3, nY3+25, 85, nHeight, Label = self.dUI.get('nbsp2', "#error#"), State = True)
        self.nbsp3 = self._addWidget('nbsp3', 'CheckBox', nX3, nY3+35, nWidth, nHeight, Label = self.dUI.get('nbsp3', "#error#"), State = True)
        self.nbsp4 = self._addWidget('nbsp4', 'CheckBox', nX3, nY3+45, 85, nHeight, Label = self.dUI.get('nbsp4', "#error#"), State = True)
        self.nnbsp1 = self._addWidget('nnbsp1', 'CheckBox', nX3+85, nY3+15, 30, nHeight, Label = self.dUI.get('nnbsp', "#error#"), State = False)
        self.nnbsp2 = self._addWidget('nnbsp2', 'CheckBox', nX3+85, nY3+25, 30, nHeight, Label = self.dUI.get('nnbsp', "#error#"), State = False)
        self.nnbsp4 = self._addWidget('nnbsp4', 'CheckBox', nX3+85, nY3+45, 30, nHeight, Label = self.dUI.get('nnbsp', "#error#"), State = False)
        self.nbsp1_res = self._addWidget('nbsp1_res', 'FixedText', nPosRes, nY3+15, 20, nHeight, Label = "", Align = 2)
        self.nbsp2_res = self._addWidget('nbsp2_res', 'FixedText', nPosRes, nY3+25, 20, nHeight, Label = "", Align = 2)
        self.nbsp3_res = self._addWidget('nbsp3_res', 'FixedText', nPosRes, nY3+35, 20, nHeight, Label = "", Align = 2)
        self.nbsp4_res = self._addWidget('nbsp4_res', 'FixedText', nPosRes, nY3+45, 20, nHeight, Label = "", Align = 2)
        
        # group box // typographical marks
        nX4 = 168; nY4 = 5
        nPosRes = nX4 + nGroupBoxWith - 32
        gbm4 = self._addWidget('groupbox4', 'GroupBox', nX4-5, nY4, nGroupBoxWith, 100, Label = "  " * len(self.dUI.get('typo', "#error#")), FontDescriptor = xFD1)
        self.typo = self._addWidget('typo', 'CheckBox', nX4, nY4+2, nWidth, nHeight, Label = self.dUI.get('typo', "#error#"), FontDescriptor = xFD1, FontRelief = 1, TextColor = nColor, State = True)
        self.typo1 = self._addWidget('typo1', 'CheckBox', nX4, nY4+15, nWidth, nHeight, Label = self.dUI.get('typo1', "#error#"), State = True)
        self.typo2 = self._addWidget('typo2', 'CheckBox', nX4, nY4+25, nWidth, nHeight, Label = self.dUI.get('typo2', "#error#"), State = True)
        self.typo3 = self._addWidget('typo3', 'CheckBox', nX4, nY4+35, nWidth, nHeight, Label = self.dUI.get('typo3', "#error#"), State = True)
        self.typo3a = self._addWidget('typo3a', 'RadioButton', nX4+10, nY4+45, nWidthHalf, nHeight, Label = self.dUI.get('emdash', "#error#"))
        self.typo3b = self._addWidget('typo3b', 'RadioButton', nX4+70, nY4+45, nWidthHalf, nHeight, Label = self.dUI.get('endash', "#error#"), State = True)
        self.typo4 = self._addWidget('typo4', 'CheckBox', nX4, nY4+55, nWidth, nHeight, Label = self.dUI.get('typo4', "#error#"), State = True)
        self.typo4a = self._addWidget('typo4a', 'RadioButton', nX4+10, nY4+65, nWidthHalf, nHeight, Label = self.dUI.get('emdash', "#error#"), State = True)
        self.typo4b = self._addWidget('typo4b', 'RadioButton', nX4+70, nY4+65, nWidthHalf, nHeight, Label = self.dUI.get('endash', "#error#"))
        self.typo5 = self._addWidget('typo5', 'CheckBox', nX4, nY4+75, nWidth, nHeight, Label = self.dUI.get('typo5', "#error#"), State = True)
        self.typo6 = self._addWidget('typo6', 'CheckBox', nX4, nY4+85, nWidth, nHeight, Label = self.dUI.get('typo6', "#error#"), State = True)
        self.typo1_res = self._addWidget('typo1_res', 'FixedText', nPosRes, nY4+15, 20, nHeight, Label = "", Align = 2)
        self.typo2_res = self._addWidget('typo2_res', 'FixedText', nPosRes, nY4+25, 20, nHeight, Label = "", Align = 2)
        self.typo3_res = self._addWidget('typo3_res', 'FixedText', nPosRes, nY4+35, 20, nHeight, Label = "", Align = 2)
        self.typo4_res = self._addWidget('typo4_res', 'FixedText', nPosRes, nY4+55, 20, nHeight, Label = "", Align = 2)
        self.typo5_res = self._addWidget('typo5_res', 'FixedText', nPosRes, nY4+75, 20, nHeight, Label = "", Align = 2)
        self.typo6_res = self._addWidget('typo6_res', 'FixedText', nPosRes, nY4+85, 20, nHeight, Label = "", Align = 2)
        
        # group box // misc.
        nX5 = 168; nY5 = 110
        gbm5 = self._addWidget('groupbox5', 'GroupBox', nX5-5, nY5, nGroupBoxWith, 80, Label = "  " * len(self.dUI.get('misc', "#error#")), FontDescriptor = xFD1)
        self.misc = self._addWidget('misc', 'CheckBox', nX5, nY5+2, nWidth, nHeight, Label = self.dUI.get('misc', "#error#"), FontDescriptor = xFD1, FontRelief = 1, TextColor = nColor, State = True)
        self.misc1 = self._addWidget('misc1', 'CheckBox', nX5, nY5+15, 85, nHeight, Label = self.dUI.get('misc1', "#error#"), State = True)
        self.misc1a = self._addWidget('misc1a', 'CheckBox', nX5+85, nY5+15, 25, nHeight, Label = self.dUI.get('misc1a', "#error#"), State = True)
        self.misc2 = self._addWidget('misc2', 'CheckBox', nX5, nY5+25, nWidth, nHeight, Label = self.dUI.get('misc2', "#error#"), State = True)
        self.misc3 = self._addWidget('misc3', 'CheckBox', nX5, nY5+35, nWidth, nHeight, Label = self.dUI.get('misc3', "#error#"), State = True)
        self.misc4 = self._addWidget('misc4', 'CheckBox', nX5, nY5+45, nWidth, nHeight, Label = self.dUI.get('misc4', "#error#"), State = True)
        self.misc5 = self._addWidget('misc5', 'CheckBox', nX5, nY5+55, nWidth, nHeight, Label = self.dUI.get('misc5', "#error#"), State = True)
        self.misc5b = self._addWidget('misc5b', 'CheckBox', nX5+10, nY5+65, nWidth-40, nHeight, Label = self.dUI.get('misc5b', "#error#"), State = False)
        self.misc5c = self._addWidget('misc5c', 'CheckBox', nX5+nWidth-25, nY5+65, 30, nHeight, Label = self.dUI.get('misc5c', "#error#"), State = False)
        self.misc1_res = self._addWidget('misc1_res', 'FixedText', nPosRes, nY5+15, 20, nHeight, Label = "", Align = 2)
        self.misc2_res = self._addWidget('misc2_res', 'FixedText', nPosRes, nY5+25, 20, nHeight, Label = "", Align = 2)
        self.misc3_res = self._addWidget('misc3_res', 'FixedText', nPosRes, nY5+35, 20, nHeight, Label = "", Align = 2)
        self.misc4_res = self._addWidget('misc4_res', 'FixedText', nPosRes, nY5+45, 20, nHeight, Label = "", Align = 2)
        self.misc5_res = self._addWidget('misc5_res', 'FixedText', nPosRes, nY5+55, 20, nHeight, Label = "", Align = 2)
        
        # group box // restructuration
        nX6 = 168; nY6 = 195
        gbm6 = self._addWidget('groupbox6', 'GroupBox', nX6-5, nY6, nGroupBoxWith, 50, Label = "  " * len(self.dUI.get('struct', "#error#")), FontDescriptor = xFD1)
        self.struct = self._addWidget('struct', 'CheckBox', nX6, nY6+2, nWidth, nHeight, Label = self.dUI.get('struct', "#error#"), FontDescriptor = xFD1, FontRelief = 1, TextColor = nColor, State = False)
        self.struct1 = self._addWidget('struct1', 'CheckBox', nX6, nY6+15, nWidth, nHeight, Label = self.dUI.get('struct1', "#error#"), State = True, Enabled = False)
        self.struct2 = self._addWidget('struct2', 'CheckBox', nX6, nY6+25, nWidth, nHeight, Label = self.dUI.get('struct2', "#error#"), State = True, Enabled = False)
        self.struct3 = self._addWidget('struct3', 'CheckBox', nX6, nY6+35, nWidth, nHeight, Label = self.dUI.get('struct3', "#error#"), HelpText = self.dUI.get('struct3_help', "#error#"),State = False, Enabled = False)
        self.struct1_res = self._addWidget('struct1_res', 'FixedText', nPosRes, nY6+15, 20, nHeight, Label = "", Align = 2)
        self.struct2_res = self._addWidget('struct2_res', 'FixedText', nPosRes, nY6+25, 20, nHeight, Label = "", Align = 2)
        self.struct3_res = self._addWidget('struct3_res', 'FixedText', nPosRes, nY6+35, 20, nHeight, Label = "", Align = 2)
        # dialog height
        self.xDialog.Height = 270

        # lists of checkbox widgets
        self.dCheckboxWidgets = {
            "ssp":      [self.ssp1, self.ssp2, self.ssp3, self.ssp4, self.ssp5, self.ssp6, self.ssp7],
            "space":    [self.space1, self.space2],
            "nbsp":     [self.nbsp1, self.nbsp2, self.nbsp3, self.nbsp4, self.nnbsp1, self.nnbsp2, self.nnbsp4],
            "typo":     [self.typo1, self.typo2, self.typo3, self.typo3a, self.typo3b, self.typo4, self.typo4a, self.typo4b, self.typo5, self.typo6],
            "misc":     [self.misc1, self.misc2, self.misc3, self.misc4, self.misc5, self.misc1a, self.misc5b, self.misc5c],
            "struct":   [self.struct1, self.struct2, self.struct3]
        }

        # progress bar
        self.pbar = self._addWidget('pbar', 'ProgressBar', 22, self.xDialog.Height-16, 215, 10)
        self.pbar.ProgressValueMin = 0
        self.pbar.ProgressValueMax = 26
        # time counter
        self.time_res = self._addWidget('time_res', 'FixedText', 240, self.xDialog.Height-15, 20, nHeight, Label = "", Align = 2)

        # buttons
        self.bdefault = self._addWidget('default', 'Button', 5, self.xDialog.Height-19, 15, 15, Label = self.dUI.get('default', "#error#"), HelpText = self.dUI.get('default_help', "#error#"), FontDescriptor = xFD2, TextColor = 0x444444)
        #self.bhelp = self._addWidget('help', 'Button', self.xDialog.Width-125, self.xDialog.Height-20, 10, 15, Label = "?", FontDescriptor = xFD2, TextColor = 0x444444)
        #self.bsel = self._addWidget('bsel', 'CheckBox', nX6, self.xDialog.Height-40, nWidth-55, nHeight, Label = self.dUI.get('bsel', "#error#"))        
        self.bapply = self._addWidget('apply', 'Button', self.xDialog.Width-55, self.xDialog.Height-19, 50, 15, Label = self.dUI.get('apply', "#error#"), FontDescriptor = xFD2, TextColor = 0x004400)
        self.infomsg = self._addWidget('infomsg', 'FixedText', nX1-5, self.xDialog.Height-60, nGroupBoxWith, 50, Label = self.dUI.get("warningmsg", "## ERROR ##"), FontDescriptor = xFDsmall, MultiLine = True, TextColor = 0x333333)

        # load configuration
        self._loadConfig()

        ## container
        self.xContainer = self.xSvMgr.createInstanceWithContext('com.sun.star.awt.UnoControlDialog', self.ctx)
        self.xContainer.setModel(self.xDialog)
        self.xContainer.setVisible(False)
        #self.xContainer.getControl('help').addActionListener(self)
        #self.xContainer.getControl('help').setActionCommand('Help')
        self.xContainer.getControl('default').addActionListener(self)
        self.xContainer.getControl('default').setActionCommand('Default')
        self.xContainer.getControl('apply').addActionListener(self)
        self.xContainer.getControl('apply').setActionCommand('Apply')
        self.xContainer.getControl('ssp').addActionListener(self)
        self.xContainer.getControl('ssp').setActionCommand('SwitchSsp')
        self.xContainer.getControl('space').addActionListener(self)
        self.xContainer.getControl('space').setActionCommand('SwitchSpace')
        self.xContainer.getControl('nbsp').addActionListener(self)
        self.xContainer.getControl('nbsp').setActionCommand('SwitchNbsp')
        self.xContainer.getControl('typo').addActionListener(self)
        self.xContainer.getControl('typo').setActionCommand('SwitchTypo')
        self.xContainer.getControl('misc').addActionListener(self)
        self.xContainer.getControl('misc').setActionCommand('SwitchMisc')
        self.xContainer.getControl('struct').addActionListener(self)
        self.xContainer.getControl('struct').setActionCommand('SwitchStruct')
        xToolkit = self.xSvMgr.createInstanceWithContext('com.sun.star.awt.ExtToolkit', self.ctx)
        self.xContainer.createPeer(xToolkit, None)
        self.xContainer.execute()
        
    def actionPerformed (self, xActionEvent):
        try:
            if xActionEvent.ActionCommand == 'Apply':
                if self.bClose:
                    self.xContainer.endExecute()
                else:
                    xDesktop = self.ctx.ServiceManager.createInstanceWithContext("com.sun.star.frame.Desktop", self.ctx)
                    xElem = xDesktop.getCurrentComponent()

                    # Writer
                    if True:
                        # no selection
                        self._saveConfig()
                        self._replaceAll(xElem)
                    else:
                        # modify selected text only
                        pass    
                        #xSelecList = xDoc.getCurrentSelection()
                        #xSel = xSelecList.getByIndex(0)

                    # Impress
                    # Note: impossible to format text on Impress right now as ReplaceDescriptors don’t accept regex!
                    #xPages = xElem.getDrawPages()
                    #for i in range(xPages.Count):
                    #    self._replaceAll(xPages.getByIndex(i))
                    #xPages = xElem.getMasterPages()
                    #for i in range(xPages.Count):
                    #    self._replaceAll(xPages.getByIndex(i))
                    self._setApplyButtonLabel()
            elif xActionEvent.ActionCommand == 'SwitchSsp':
                self._switchCheckBox(self.ssp)
                self._setApplyButtonLabel()
            elif xActionEvent.ActionCommand == 'SwitchSpace':
                self._switchCheckBox(self.space)
                self._setApplyButtonLabel()
            elif xActionEvent.ActionCommand == 'SwitchNbsp':
                self._switchCheckBox(self.nbsp)
                self._setApplyButtonLabel()
            elif xActionEvent.ActionCommand == 'SwitchTypo':
                self._switchCheckBox(self.typo)
                self._setApplyButtonLabel()
            elif xActionEvent.ActionCommand == 'SwitchMisc':
                self._switchCheckBox(self.misc)
                self._setApplyButtonLabel()
            elif xActionEvent.ActionCommand == 'SwitchStruct':
                self._switchCheckBox(self.struct)
                self._setApplyButtonLabel()
            elif xActionEvent.ActionCommand == 'Default':
                self._setConfig(tf_options.dDefaultOpt)
                self._setApplyButtonLabel()
            elif xActionEvent.ActionCommand == 'Help':
                pass
                # xDesktop = self.xSvMgr.createInstanceWithContext('com.sun.star.frame.Desktop', self.ctx)
                # xDoc = xDesktop.getCurrentComponent()
                # xWindow = xDoc.CurrentController.Frame.ContainerWindow
                # MessageBox (xWindow, self.dUI.get("warningmsg", "## ERROR ##"), self.dUI.get("warningtitle", "## ERROR ##"), "warningbox")
            else:
                print("Wrong command: " + xActionEvent.ActionCommand)
        except:
            traceback.print_exc()

    def _loadConfig (self):
        try:
            if sys.version_info.major == 3:
                imp.reload(tf_options)
            else:
                reload(tf_options)
            self._setConfig(tf_options.dOpt)
        except:
            traceback.print_exc()

    def _setConfig (self, d):
        try:
            for key, val in d.items():
                w = getattr(self, key)
                w.State = val
                if key in self.dCheckboxWidgets:
                    self._switchCheckBox(w)
        except:
            raise

    def _saveConfig (self):
        try:
            # create options dictionary
            dOpt = {}
            for key, lWidget in self.dCheckboxWidgets.items():
                w = getattr(self, key)
                dOpt[w.Name] = w.State
                for w in lWidget:
                    dOpt[w.Name] = w.State
            # get extension path
            xDefaultContext = self.ctx.ServiceManager.DefaultContext
            xPackageInfoProvider = xDefaultContext.getValueByName("/singletons/com.sun.star.deployment.PackageInformationProvider")
            sExtPath = xPackageInfoProvider.getPackageLocation("French.linguistic.resources.from.Dicollecte.by.OlivierR")
            sExtPath = sExtPath[8:] + "/pythonpath/tf_options.py"  # remove "file:///"
            sExtPath = os.path.abspath(sExtPath)
            # write file
            if os.path.isfile(sExtPath):
                hOpt = open(sExtPath, "w")
                hOpt.write("dDefaultOpt = " + str(tf_options.dDefaultOpt) + "\n")
                hOpt.write("dOpt = " + str(dOpt))
                hOpt.close()
        except:
            traceback.print_exc()

    def _switchCheckBox (self, wGroupCheckbox):
        for w in self.dCheckboxWidgets.get(wGroupCheckbox.Name, []):
            w.Enabled = wGroupCheckbox.State

    def _setApplyButtonLabel (self):
        if self.ssp.State or self.space.State or self.nbsp.State or self.typo.State or self.misc.State or self.struct.State:
            self.bClose = False
            self.bapply.Label = self.dUI.get('apply', "#error#")
            self.bapply.TextColor = 0x004400
        else:
            self.bClose = True
            self.bapply.Label = self.dUI.get('close', "#error#")
            self.bapply.TextColor = 0x440000
        self.xContainer.setVisible(True)

    def _replaceAll (self, xElem):
        try:
            nStartTime = time.clock()
            self.xContainer.setVisible(True)
            # change pointer
            xPointer = self.ctx.ServiceManager.createInstanceWithContext("com.sun.star.awt.Pointer", self.ctx)
            xPointer.setType(uno.getConstantByName("com.sun.star.awt.SystemPointer.WAIT"))
            xWindowPeer = self.xContainer.getPeer()
            xWindowPeer.setPointer(xPointer)
            for x in xWindowPeer.Windows:
                x.setPointer(xPointer)
            # NOTE: A LOT OF REGEX COULD BE MERGED IF ICU ENGINE WAS NOT SO BUGGY
            self.pbar.ProgressValue = 0
            # Restructuration
            if self.struct.State:
                if self.struct1.State:
                    n = self._replaceText(xElem, "\\n", "\\n", True) # end of line => end of paragraph
                    self.struct1_res.Label = str(n)
                    self.pbar.ProgressValue += 1
                if self.struct2.State:
                    n = self._replaceText(xElem, u"([:alpha:])- *\n([:alpha:])", u"$1$2", True)  # EOL
                    n += self._replaceHyphenAtEndOfParagraphs(xElem) # EOP
                    self.struct2_res.Label = str(n)
                    self.pbar.ProgressValue += 1
                if self.struct3.State:
                    n = self._mergeContiguousParagraphs(xElem)
                    self.struct3_res.Label = str(n)
                    self.pbar.ProgressValue += 1
                self.struct.State = False
                self._switchCheckBox(self.struct)
            self.pbar.ProgressValue = 3
            # espaces surnuméraires
            if self.ssp.State:
                if self.ssp3.State:
                    n = self._replaceText(xElem, u"[  ]+$", "", True)
                    self.ssp3_res.Label = str(n)
                    self.pbar.ProgressValue += 1
                if self.ssp2.State:
                    n = self._replaceText(xElem, u"  ", u" ", False) # espace + espace insécable -> espace
                    n += self._replaceText(xElem, u"  ", u" ", False) # espace insécable + espace -> espace
                    n += self._replaceText(xElem, u"  +", u" ", True) # espaces surnuméraires
                    n += self._replaceText(xElem, u"  +", u" ", True) # espaces insécables surnuméraires
                    self.ssp2_res.Label = str(n)
                    self.pbar.ProgressValue += 1
                if self.ssp1.State:
                    n = self._replaceText(xElem, u"^[  ]+", "", True)
                    self.ssp1_res.Label = str(n)
                    self.pbar.ProgressValue += 1
                if self.ssp4.State:
                    n = self._replaceText(xElem, u" +(?=[.,…])", "", True)
                    self.ssp4_res.Label = str(n)
                    self.pbar.ProgressValue += 1
                if self.ssp5.State:
                    n = self._replaceText(xElem, u"\\([  ]+", u"(", True)
                    n += self._replaceText(xElem, u"[  ]+\\)", u")", True)
                    self.ssp5_res.Label = str(n)
                    self.pbar.ProgressValue += 1
                if self.ssp6.State:
                    n = self._replaceText(xElem, u"\\[[  ]+", u"[", True)
                    n += self._replaceText(xElem, u"[  ]+\\]", u"]", True)
                    self.ssp6_res.Label = str(n)
                    self.pbar.ProgressValue += 1
                if self.ssp7.State:
                    n = self._replaceText(xElem, u"“[  ]+", u"“", True)
                    n += self._replaceText(xElem, u"[  ]”", u"”", True)
                    self.ssp7_res.Label = str(n)
                    self.pbar.ProgressValue += 1
                self.ssp.State = False
                self._switchCheckBox(self.ssp)
            self.pbar.ProgressValue = 10
            # espaces typographiques
            if self.nbsp.State:
                if self.nbsp1.State:
                    if self.nnbsp1.State:
                        # espaces insécables fines
                        n = self._replaceText(xElem, u"(?<=[:alnum:])([;?!])", u" $1", True)
                        n += self._replaceText(xElem, u"(?<=[]…)»}])([;?!])", u" $1", True)
                        n += self._replaceText(xElem, u"[  ]+([;?!])", u" $1", True)
                        n += self._replaceText(xElem, u"(?<=[:alnum:]):", u" :", True)
                        n += self._replaceText(xElem, u"(?<=[]…)»}]):", u" :", True)
                        n += self._replaceText(xElem, u"[  ]+:", u" :", True)
                    else:
                        # espaces insécables
                        n = self._replaceText(xElem, u"(?<=[:alnum:])([:;?!])", u" $1", True)
                        n += self._replaceText(xElem, u"(?<=[]…)»}])([:;?!])", u" $1", True)
                        n += self._replaceText(xElem, u"[  ]+([:;?!])", u" $1", True)
                    n -= self._replaceText(xElem, u"([[(])[   ]([!?:;])", u"$1$2", True)
                    n -= self._replaceText(xElem, u"(?<=http)[   ]:(?=//)", u":", True)
                    n -= self._replaceText(xElem, u"(?<=https)[   ]:(?=//)", u":", True)
                    n -= self._replaceText(xElem, u"(?<=ftp)[   ]:(?=//)", u":", True)
                    self.nbsp1_res.Label = str(n)
                    self.pbar.ProgressValue += 1
                if self.nbsp2.State:
                    if self.nnbsp2.State:
                        # espaces insécables fines
                        n = self._replaceText(xElem, u"«(?=[:alnum:])", u"« ", True)
                        n += self._replaceText(xElem, u"«[  ]+", u"« ", True)
                        n += self._replaceText(xElem, u"(?<=[:alnum:]|[.!?])»", u" »", True)
                        n += self._replaceText(xElem, u"[  ]+»", u" »", True)
                    else:
                        # espaces insécables
                        n = self._replaceText(xElem, u"«(?=[:alnum:])", u"« ", True)
                        n += self._replaceText(xElem, u"«[  ]+", u"« ", True)
                        n += self._replaceText(xElem, u"(?<=[:alnum:]|[.!?])»", u" »", True)
                        n += self._replaceText(xElem, u"[  ]+»", u" »", True)
                    self.nbsp2_res.Label = str(n)
                    self.pbar.ProgressValue += 1
                if self.nbsp3.State:
                    n = self._replaceText(xElem, u"([:digit:])([%‰€$£¥˚Ω])", u"$1 $2", True)
                    n += self._replaceText(xElem, u"([:digit:]) ([%‰€$£¥˚Ω])", u"$1 $2", True)
                    self.nbsp3_res.Label = str(n)
                    self.pbar.ProgressValue += 1
                if self.nbsp4.State:
                    if self.nnbsp4.State:
                        # espaces insécables fines
                        n = self._replaceText(xElem, u"([:digit:])[  ]([:digit:])", u"$1 $2", True)
                    else:
                        # espaces insécables
                        n = self._replaceText(xElem, u"([:digit:])[  ]([:digit:])", u"$1 $2", True)
                    self.nbsp4_res.Label = str(n)
                    self.pbar.ProgressValue += 1
                if False:
                    n = self._replaceText(xElem, u"\\b(MM?\\.|Mlle|Mgr) ", u"$1 ", True)
                    self.nbsp3_res.Label = str(n)
                    self.pbar.ProgressValue += 1
                self.nbsp.State = False
                self._switchCheckBox(self.nbsp)
            self.pbar.ProgressValue = 14
            # espaces manquants
            if self.typo.State:
                if self.typo6.State:
                    n = self._replaceText(xElem, u"\\bN\\.([ms])\\b", u"N·$1", True, True) # N·m et N·m-1, N·s
                    n += self._replaceText(xElem, u"\\bW\\.h\\b", u"W·h", True, True)
                    n += self._replaceText(xElem, u"\\bPa\\.s\\b", u"Pa·s", True, True)
                    n += self._replaceText(xElem, u"\\bA\\.h\\b", u"A·h", True, True)
                    n += self._replaceText(xElem, u"\\bΩ\\.m\\b", u"Ω·m", True, True)
                    n += self._replaceText(xElem, u"\\bS\\.m\\b", u"S·m", True, True)
                    n += self._replaceText(xElem, u"\\bg\\.s(?=-1)\\b", u"g·s", True, True)
                    n += self._replaceText(xElem, u"\\bm\\.s(?=-[12])\\b", u"m·s", True, True)
                    n += self._replaceText(xElem, u"\\bg\\.m(?=2|-3)\\b", u"g·m", True, True)
                    n += self._replaceText(xElem, u"\\bA\\.m(?=-1)\\b", u"A·m", True, True)
                    n += self._replaceText(xElem, u"\\bJ\\.K(?=-1)\\b", u"J·K", True, True)
                    n += self._replaceText(xElem, u"\\bW\\.m(?=-2)\\b", u"W·m", True, True)
                    n += self._replaceText(xElem, u"\\bcd\\.m(?=-2)\\b", u"cd·m", True, True)
                    n += self._replaceText(xElem, u"\\bC\\.kg(?=-1)\\b", u"C·kg", True, True)
                    n += self._replaceText(xElem, u"\\bH\\.m(?=-1)\\b", u"H·m", True, True)
                    n += self._replaceText(xElem, u"\\bJ\\.kg(?=-1)\\b", u"J·kg", True, True)
                    n += self._replaceText(xElem, u"\\bJ\\.m(?=-3)\\b", u"J·m", True, True)
                    n += self._replaceText(xElem, u"\\bm[2²]\\.s\\b", u"m²·s", True, True)
                    n += self._replaceText(xElem, u"\\bm[3³]\\.s(?=-1)\\b", u"m³·s", True, True)
                    #n += self._replaceText(xElem, u"\\bJ.kg-1.K-1\\b", u"J·kg-1·K-1", True, True)
                    #n += self._replaceText(xElem, u"\\bW.m-1.K-1\\b", u"W·m-1·K-1", True, True)
                    #n += self._replaceText(xElem, u"\\bW.m-2.K-1\\b", u"W·m-2·K-1", True, True)
                    self.typo6_res.Label = str(n)
                    self.pbar.ProgressValue += 1
            if self.space.State:
                if self.space1.State:
                    # ICU is buggy: "([;?!…])(?=[:alnum:])" => "$1 " doesn’t work properly
                    n = self._replaceText(xElem, u";(?=[:alnum:])", u"; ", True)
                    n += self._replaceText(xElem, u"\\?(?=[:alnum:])", u"? ", True)
                    n += self._replaceText(xElem, u"!(?=[:alnum:])", u"! ", True)
                    n += self._replaceText(xElem, u"…(?=[:alnum:])", u"… ", True)
                    n += self._replaceText(xElem, u"\\.(?=[A-ZÉÈÎ][:alpha:])", u". ", True, True)
                    n += self._replaceText(xElem, u"\\.(?=À)", u". ", True, True)
                    n += self._replaceText(xElem, u",(?=[:alpha:])", u", ", True)
                    n += self._replaceText(xElem, u":(?=[:alpha:])", u": ", True)
                    # exceptions:
                    n -= self._replaceText(xElem, u"(?<=DnT), w\\b", u",w", True, True)
                    n -= self._replaceText(xElem, u"(?<=DnT), A\\b", u",A", True, True)
                    self.space1_res.Label = str(n)
                    self.pbar.ProgressValue += 1
                if self.space2.State:
                    n = self._replaceText(xElem, u" -(?=[:alpha:]|[\"«“'‘])", u" - ", True)
                    n += self._replaceText(xElem, u" –(?=[:alpha:]|[\"«“'‘])", u" – ", True) # demi-cadratin
                    n += self._replaceText(xElem, u" —(?=[:alpha:]|[\"«“'‘])", u" — ", True) # cadratin
                    n += self._replaceText(xElem, u"(?<=[:alpha:])([–—-]) ", u" $1 ", True)
                    n += self._replaceText(xElem, u"(?<=[\"»”'’])([–—-]) ", u" $1 ", True)
                    self.space2_res.Label = str(n)
                    self.pbar.ProgressValue += 1
                self.space.State = False
                self._switchCheckBox(self.space)
            self.pbar.ProgressValue = 16
            # signes typographiques
            if self.typo.State:
                if self.typo1.State:
                    n = self._replaceText(xElem, u"\\bl['´](?=[:alnum:])", u"l’", True, True)
                    n += self._replaceText(xElem, u"\\bj['´](?=[:alnum:])", u"j’", True, True)
                    n += self._replaceText(xElem, u"\\bm['´](?=[:alnum:])", u"m’", True, True)
                    n += self._replaceText(xElem, u"\\bt['´](?=[:alnum:])", u"t’", True, True)
                    n += self._replaceText(xElem, u"\\bs['´](?=[:alnum:])", u"s’", True, True)
                    n += self._replaceText(xElem, u"\\bc['´](?=[:alnum:])", u"c’", True, True)
                    n += self._replaceText(xElem, u"\\bd['´](?=[:alnum:])", u"d’", True, True)
                    n += self._replaceText(xElem, u"\\bn['´](?=[:alnum:])", u"n’", True, True)
                    n += self._replaceText(xElem, u"\\bç['´](?=[:alnum:])", u"ç’", True, True)
                    n += self._replaceText(xElem, u"\\bL['´](?=[:alnum:])", u"L’", True, True)
                    n += self._replaceText(xElem, u"\\bJ['´](?=[:alnum:])", u"J’", True, True)
                    n += self._replaceText(xElem, u"\\bM['´](?=[:alnum:])", u"M’", True, True)
                    n += self._replaceText(xElem, u"\\bT['´](?=[:alnum:])", u"T’", True, True)
                    n += self._replaceText(xElem, u"\\bS['´](?=[:alnum:])", u"S’", True, True)
                    n += self._replaceText(xElem, u"\\bC['´](?=[:alnum:])", u"C’", True, True)
                    n += self._replaceText(xElem, u"\\bD['´](?=[:alnum:])", u"D’", True, True)
                    n += self._replaceText(xElem, u"\\bN['´](?=[:alnum:])", u"N’", True, True)
                    n += self._replaceText(xElem, u"\\bÇ['´](?=[:alnum:])", u"Ç’", True, True)
                    n += self._replaceText(xElem, u"\\b(qu|jusqu|lorsqu|puisqu|quoiqu|quelqu|presqu|entr|aujourd)['´]", u"$1’", True)
                    #n = self._replaceText(xElem, u"\\b([ljmtscdn]|qu|jusqu|lorsqu|puisqu|quoiqu|quelqu|presqu|entr|aujourd)['´]", u"$1’", True)
                    n += self._replaceText(xElem, u"((d|qu)’aujourd)['´]", u"$1’", True)
                    self.typo1_res.Label = str(n)
                    self.pbar.ProgressValue += 1
                if self.typo2.State:
                    n = self._replaceText(xElem, u"...", u"…", False)
                    n += self._replaceText(xElem, u"(?<=…)[.][.]", u"…", True)
                    n += self._replaceText(xElem, u"…[.](?![.])", u"…", True)
                    self.typo2_res.Label = str(n)
                    self.pbar.ProgressValue += 1
                if self.typo3.State:
                    if self.typo3b.State:
                        # demi-cadratin
                        n = self._replaceText(xElem, u" - ", u" – ", False)
                        n += self._replaceText(xElem, u" — ", u" – ", False)
                        n += self._replaceText(xElem, u" -,", u" –,", False)
                        n += self._replaceText(xElem, u" —,", u" –,", False)
                    else:
                        # cadratin
                        n = self._replaceText(xElem, u" - ", u" — ", False)
                        n += self._replaceText(xElem, u" – ", u" — ", False)
                        n += self._replaceText(xElem, u" -,", u" —,", False)
                        n += self._replaceText(xElem, u" –,", u" —,", False)
                    self.typo3_res.Label = str(n)
                    self.pbar.ProgressValue += 1
                if self.typo4.State:
                    if self.typo4a.State:
                        # cadratin
                        n = self._replaceText(xElem, u"^-[  ]", u"— ", True)
                        n += self._replaceText(xElem, u"^–[  ]", u"— ", True)
                        n += self._replaceText(xElem, u"^— ", u"— ", True)
                        n += self._replaceText(xElem, u"^[-–—](?=[:alnum:])", u"— ", True)
                    else:
                        # demi-cadratin
                        n = self._replaceText(xElem, u"^-[  ]", u"– ", True)
                        n += self._replaceText(xElem, u"^—[  ]", u"– ", True)
                        n += self._replaceText(xElem, u"^– ", u"– ", True)
                        n += self._replaceText(xElem, u"^[-–—](?=[:alnum:])", u"– ", True)
                    self.typo4_res.Label = str(n)
                    self.pbar.ProgressValue += 1
                if self.typo5.State:
                    n = self._replaceText(xElem, u'"([:alpha:]+)"', u"“$1”", True)
                    n += self._replaceText(xElem, u"''([:alpha:]+)''", u"“$1”", True)
                    n += self._replaceText(xElem, u"'([:alpha:]+)'", u"“$1”", True)
                    n += self._replaceText(xElem, u'^"(?=[:alnum:])', u"« ", True)
                    n += self._replaceText(xElem, u"^''(?=[:alnum:])", u"« ", True)
                    n += self._replaceText(xElem, u' "(?=[:alnum:])', u" « ", True)
                    n += self._replaceText(xElem, u" ''(?=[:alnum:])", u" « ", True)
                    n += self._replaceText(xElem, u'\\("(?=[:alnum:])', u"(« ", True)
                    n += self._replaceText(xElem, u"\\(''(?=[:alnum:])", u"(« ", True)
                    n += self._replaceText(xElem, u'(?<=[:alnum:])"$', u" »", True)
                    n += self._replaceText(xElem, u"(?<=[:alnum:])''$", u" »", True)
                    n += self._replaceText(xElem, u'(?<=[:alnum:])"(?=[] ,.:;?!…)])', u" »", True)
                    n += self._replaceText(xElem, u"(?<=[:alnum:])''(?=[] ,.:;?!…)])", u" »", True)
                    n += self._replaceText(xElem, u'(?<=[.!?…])" ', u" » ", True)
                    n += self._replaceText(xElem, u'(?<=[.!?…])"$', u" »", True)
                    self.typo5_res.Label = str(n)
                    self.pbar.ProgressValue += 1
                self.typo.State = False
                self._switchCheckBox(self.typo)
            self.pbar.ProgressValue = 22
            # divers
            if self.misc.State:
                if self.misc1.State:
                    if self.misc1a.State:
                        n = self._replaceText(xElem, u"(?<=\\b[0-9][0-9][0-9][0-9])(i?[èe]me|è|e)\\b", u"ᵉ", True)
                        n += self._replaceText(xElem, u"(?<=\\b[0-9][0-9][0-9])(i?[èe]me|è|e)\\b", u"ᵉ", True)
                        n += self._replaceText(xElem, u"(?<=\\b[0-9][0-9])(i?[èe]me|è|e)\\b", u"ᵉ", True)
                        n += self._replaceText(xElem, u"(?<=\\b[0-9])(i?[èe]me|è|e)\\b", u"ᵉ", True)
                        n += self._replaceText(xElem, u"(?<=\\b[XVICL][XVICL][XVICL][XVICL])(i?[èe]me|è|e)\\b", u"ᵉ", True, True)
                        n += self._replaceText(xElem, u"(?<=\\b[XVICL][XVICL][XVICL])(i?[èe]me|è|e)\\b", u"ᵉ", True, True)
                        n += self._replaceText(xElem, u"(?<=\\b[XVICL][XVICL])(i?[èe]me|è|e)\\b", u"ᵉ", True, True)
                        n += self._replaceText(xElem, u"(?<=\\b[XVICL])(i?[èe]me|è)\\b", u"ᵉ", True, True)
                        n += self._replaceText(xElem, u"(?<=\\b(au|l[ea]|du) [XVICL])e\\b", u"ᵉ", True, True)
                        n += self._replaceText(xElem, u"(?<=\\b[XVI])e(?= siècle)", u"ᵉ", True, True)
                        n += self._replaceText(xElem, u"(?<=\\b[1I])er\\b", u"ᵉʳ", True, True)
                        n += self._replaceText(xElem, u"(?<=\\b[1I])re\\b", u"ʳᵉ", True, True)
                        
                    else:
                        n = self._replaceText(xElem, u"(?<=\\b[0-9][0-9][0-9][0-9])(i?[èe]me|è|ᵉ)\\b", u"e", True)
                        n += self._replaceText(xElem, u"(?<=\\b[0-9][0-9][0-9])(i?[èe]me|è|ᵉ)\\b", u"e", True)
                        n += self._replaceText(xElem, u"(?<=\\b[0-9][0-9])(i?[èe]me|è|ᵉ)\\b", u"e", True)
                        n += self._replaceText(xElem, u"(?<=\\b[0-9])(i?[èe]me|è|ᵉ)\\b", u"e", True)
                        n += self._replaceText(xElem, u"(?<=\\b[XVICL][XVICL][XVICL][XVICL])(i?[èe]me|è|ᵉ)\\b", u"e", True, True)
                        n += self._replaceText(xElem, u"(?<=\\b[XVICL][XVICL][XVICL])(i?[èe]me|è|ᵉ)\\b", u"e", True, True)
                        n += self._replaceText(xElem, u"(?<=\\b[XVICL][XVICL])(i?[èe]me|è|ᵉ)\\b", u"e", True, True)
                        n += self._replaceText(xElem, u"(?<=\\b[XVICL])(i?[èe]me|è|ᵉ)\\b", u"e", True, True)
                        n += self._replaceText(xElem, u"(?<=\\b[1I])ᵉʳ\\b", u"er", True, True)
                        n += self._replaceText(xElem, u"(?<=\\b[1I])ʳᵉ\\b", u"er", True, True)
                    self.misc1_res.Label = str(n)
                    self.pbar.ProgressValue += 1
                if self.misc2.State:
                    n = self._replaceText(xElem, u"etc(…|[.][.][.]?)", u"etc.", True, True)
                    n += self._replaceText(xElem, u"(?<!,) etc[.]", u", etc.", True, True)
                    self.misc2_res.Label = str(n)
                    self.pbar.ProgressValue += 1
                if self.misc3.State:
                    n = self._replaceText(xElem, u"[ -]t[’'](?=il|elle|on)", u"-t-", True)
                    n += self._replaceText(xElem, u" t-(?=il|elle|on)", u"-t-", True)
                    n += self._replaceText(xElem, u"[ -]t[’'-](?=ils|elles)", u"-", True)
                    n += self._replaceText(xElem, u"(?<=[td])-t-(?=il|elle|on)", u"-", True)
                    n += self._replaceText(xElem, u"(celles?|celui|ceux) (ci|là)", u"$1-$2", True)
                    n += self._replaceText(xElem, u"dix (sept|huit|neuf)", u"dix-$1", True)
                    n += self._replaceText(xElem, u"quatre vingt", u"quatre-vingt", False)
                    n += self._replaceText(xElem, u"(soixante|quatre-vingt) dix", u"$1-dix", True)
                    n += self._replaceText(xElem, u"(vingt|trente|quarante|cinquante|soixante(?:-dix|)|quatre-vingt(?:-dix|)) (deux|trois|quatre|cinq|six|sept|huit|neuf)", u"$1-$2", True)
                    n += self._replaceText(xElem, u"(?<!-)(ci) (joint|desso?us|contre|devant|avant|après|incluse|g[îi]t|gisent)", u"$1-$2", True)
                    n += self._replaceText(xElem, u"vis à vis", u"vis-à-vis", False, True)
                    n += self._replaceText(xElem, u"Vis à vis", u"Vis-à-vis", False, True)
                    n += self._replaceText(xElem, u"week end", u"week-end", False, True)
                    n += self._replaceText(xElem, u"Week end", u"Week-end", False, True)
                    n += self._replaceText(xElem, u"(plus|moins) value", u"$1-value", True)
                    self.misc3_res.Label = str(n)
                    self.pbar.ProgressValue += 1
                if self.misc4.State:
                    # no initial cap
                    n = self._replaceText(xElem, u"coeur", u"cœur", False, True)
                    n += self._replaceText(xElem, u"coel([aeio])", u"cœl$1", True, True)
                    n += self._replaceText(xElem, u"choeur", u"chœur", False, True)
                    n += self._replaceText(xElem, u"foet", u"fœt", False, True)
                    n += self._replaceText(xElem, u"oeil", u"œil", False, True)
                    n += self._replaceText(xElem, u"oeno", u"œno", False, True)
                    n += self._replaceText(xElem, u"oesoph", u"œsoph", False, True)
                    n += self._replaceText(xElem, u"oestro", u"œstro", False, True)
                    n += self._replaceText(xElem, u"oeuf", u"œuf", False, True)
                    n += self._replaceText(xElem, u"oeuvr", u"œuvr", False, True)
                    n += self._replaceText(xElem, u"moeur", u"mœur", False, True)
                    n += self._replaceText(xElem, u"noeu", u"nœu", False, True)
                    n += self._replaceText(xElem, u"soeur", u"sœur", False, True)
                    n += self._replaceText(xElem, u"voeu", u"vœu", False, True)
                    n += self._replaceText(xElem, u"aequo", u"æquo", False, True)
                    # with initial cap
                    n += self._replaceText(xElem, u"Coeur", u"Cœur", False, True)
                    n += self._replaceText(xElem, u"Coel([aeio])", u"Cœl$1", True, True)
                    n += self._replaceText(xElem, u"Choeur", u"Chœur", False, True)
                    n += self._replaceText(xElem, u"Foet", u"Fœt", False, True)
                    n += self._replaceText(xElem, u"Oeil", u"Œil", False, True)
                    n += self._replaceText(xElem, u"Oeno", u"Œno", False, True)
                    n += self._replaceText(xElem, u"Oesoph", u"Œsoph", False, True)
                    n += self._replaceText(xElem, u"Oestro", u"Œstro", False, True)
                    n += self._replaceText(xElem, u"Oeuf", u"Œuf", False, True)
                    n += self._replaceText(xElem, u"Oeuvr", u"Œuvr", False, True)
                    n += self._replaceText(xElem, u"Oeur", u"Mœur", False, True)
                    n += self._replaceText(xElem, u"Oeu", u"Nœu", False, True)
                    n += self._replaceText(xElem, u"Soeur", u"Sœur", False, True)
                    n += self._replaceText(xElem, u"Voeu", u"Vœu", False, True)
                    n += self._replaceText(xElem, u"Aequo", u"Æquo", False, True)
                    self.misc4_res.Label = str(n)
                    self.pbar.ProgressValue += 1
                if self.misc5.State:
                    n = self._replaceText(xElem, u"\\b(qu|lorsqu|puisqu|quoiqu|presqu|jusqu|aujourd|entr|quelqu) ", u"$1’", True, True)
                    if self.misc5b.State:
                        n += self._replaceText(xElem, u"\\bj (?=[aàeéêiîoôuyhAÀEÉÊIÎOÔUYH])", u"j’", True, True)
                        n += self._replaceText(xElem, u"\\bn (?=[aàeéêiîoôuyhAÀEÉÊIÎOÔUYH])", u"n’", True, True)
                        n += self._replaceText(xElem, u"\\bm (?=[aàeéêiîoôuyhAÀEÉÊIÎOÔUYH])", u"m’", True, True)
                        n += self._replaceText(xElem, u"\\bt (?=[aàeéêiîoôuyhAÀEÉÊIÎOÔUYH])", u"t’", True, True)
                        n += self._replaceText(xElem, u"\\bs (?=[aàeéêiîoôuyhAÀEÉÊIÎOÔUYH])", u"s’", True, True)
                        n += self._replaceText(xElem, u"\\bc (?=[aàeéêiîoôuyhAÀEÉÊIÎOÔUYH])", u"c’", True, True)
                        n += self._replaceText(xElem, u"\\bç (?=[aàeéêiîoôuyhAÀEÉÊIÎOÔUYH])", u"ç’", True, True)
                        n += self._replaceText(xElem, u"\\bl (?=[aàeéêiîoôuyhAÀEÉÊIÎOÔUYH])", u"l’", True, True)
                        n += self._replaceText(xElem, u"\\bd (?=[aàeéêiîoôuyhAÀEÉÊIÎOÔUYH])", u"d’", True, True)
                        if self.misc5c.State:
                            n += self._replaceText(xElem, u"\\bJ (?=[aàeéêiîoôuyhAÀEÉÊIÎOÔUYH])", u"J’", True, True)
                            n += self._replaceText(xElem, u"\\bN (?=[aàeéêiîoôuyhAÀEÉÊIÎOÔUYH])", u"N’", True, True)
                            n += self._replaceText(xElem, u"\\bM (?=[aàeéêiîoôuyhAÀEÉÊIÎOÔUYH])", u"M’", True, True)
                            n += self._replaceText(xElem, u"\\bT (?=[aàeéêiîoôuyhAÀEÉÊIÎOÔUYH])", u"T’", True, True)
                            n += self._replaceText(xElem, u"\\bS (?=[aàeéêiîoôuyhAÀEÉÊIÎOÔUYH])", u"S’", True, True)
                            n += self._replaceText(xElem, u"\\bC (?=[aàeéêiîoôuyhAÀEÉÊIÎOÔUYH])", u"C’", True, True)
                            n += self._replaceText(xElem, u"\\bÇ (?=[aàeéêiîoôuyhAÀEÉÊIÎOÔUYH])", u"Ç’", True, True)
                            n += self._replaceText(xElem, u"\\bL (?=[aàeéêiîoôuyhAÀEÉÊIÎOÔUYH])", u"L’", True, True)
                            n += self._replaceText(xElem, u"\\bD (?=[aàeéêiîoôuyhAÀEÉÊIÎOÔUYH])", u"D’", True, True)
                    self.misc5_res.Label = str(n)
                    self.pbar.ProgressValue += 1
                self.misc.State = False
                self._switchCheckBox(self.misc)
            self.pbar.ProgressValue = self.pbar.ProgressValueMax
            # end of processing
            xPointer.setType(uno.getConstantByName("com.sun.star.awt.SystemPointer.ARROW"))
            xWindowPeer.setPointer(xPointer)
            for x in xWindowPeer.Windows:
                x.setPointer(xPointer)
            self.xContainer.setVisible(True) # seems necessary to refresh the dialog box and text widgets (why?)
            nEndTime = time.clock()
            print(nEndTime-nStartTime)
            self.time_res.Label = getTimeRes(nEndTime-nStartTime)
        except:
            traceback.print_exc()

    def _replaceText (self, xElem, sPattern, sRep, bRegex, bCaseSensitive=False):
        try:
            xRD = xElem.createReplaceDescriptor()
            xRD.SearchString = sPattern
            xRD.ReplaceString = sRep
            xRD.SearchRegularExpression = bRegex
            xRD.SearchCaseSensitive = bCaseSensitive
            return xElem.replaceAll(xRD)
        except:
            traceback.print_exc()

    def _replaceHyphenAtEndOfParagraphs (self, xDoc):
        self._replaceText(xDoc, u"- *$", u"-", True) # remove spaces at end of paragraphs if - is the last character
        xHunspell = self.xSvMgr.createInstanceWithContext("com.sun.star.linguistic2.SpellChecker", self.ctx)
        xCursor = xDoc.Text.createTextCursor()
        xCursor.gotoStart(False)
        n = 0
        while xCursor.gotoNextParagraph(False):
            xCursor.goLeft(2, True)
            if xCursor.String.startswith("-"):
                xCursor.gotoStartOfWord(False)
                xLocale = xCursor.CharLocale
                xCursor.gotoEndOfWord(True)
                sWord1 = xCursor.String
                xCursor.gotoNextParagraph(False)
                xCursor.gotoEndOfWord(True)
                sWord2 = xCursor.String
                if sWord1 and sWord2 and xHunspell.isValid(sWord1+sWord2, xLocale, ()):
                    xCursor.gotoStartOfParagraph(False)
                    xCursor.goLeft(2, True)
                    xCursor.setString("")
                    n += 1
            else:
                xCursor.goRight(2, False)
        return n

    def _mergeContiguousParagraphs (self, xDoc):
        self._replaceText(xDoc, u"^[  ]+$", u"", True) # clear empty paragraphs
        xCursor = xDoc.Text.createTextCursor()
        xCursor.gotoStart(False)
        n = 0
        try:
            while xCursor.gotoNextParagraph(False):
                xCursor.gotoEndOfParagraph(True)
                if xCursor.String != "":
                    xCursor.gotoStartOfParagraph(False)
                    xCursor.goLeft(1, True)
                    xCursor.setString(" ")
                    n += 1
        except:
            pass
        self._replaceText(xDoc, u"  +", u" ", True)
        return n

def getTimeRes (n):
    "returns duration in seconds as string"
    if n < 10:
        return "{:.3f} s".format(n)
    if n < 100:
        return "{:.2f} s".format(n)
    if n < 1000:
        return "{:.1f} s".format(n)
    return str(int(n)) + " s"

def getConfigSetting (sNodeConfig, bUpdate):
    # get a configuration node
    # example: aSettings = getConfigSetting( "/org.openoffice.Office.Common/Path/Current", false)
    xSvMgr = uno.getComponentContext().ServiceManager
    xConfigProvider = xSvMgr.createInstanceWithContext("com.sun.star.configuration.ConfigurationProvider", uno.getComponentContext())
    xPropertyValue = uno.createUnoStruct("com.sun.star.beans.PropertyValue")
    xPropertyValue.Name = "nodepath"
    xPropertyValue.Value = sNodeConfig
    if bUpdate:
        sService = "com.sun.star.configuration.ConfigurationUpdateAccess"
    else:
        sService = "com.sun.star.configuration.ConfigurationAccess"
    xSettings = xConfigProvider.createInstanceWithArguments(sService, (xPropertyValue,))
    return xSettings


g_ImplementationHelper = unohelper.ImplementationHelper()
g_ImplementationHelper.addImplementation(TextFormatter, 'dicollecte.TextFormatter', ('com.sun.star.task.Job',))
