# -*- encoding: UTF-8 -*-

import uno, re, sys, os, traceback
from com.sun.star.text.TextMarkupType import PROOFREADING
from com.sun.star.beans import PropertyValue
import conjfr
import mfspfr
import cregex as cr

pkg = u"grammalecte"
lang = u"fr"
locales = {'fr-MC': ['fr', 'MC', ''], 'fr-FR': ['fr', 'FR', ''], 'fr-LU': ['fr', 'LU', ''], 'fr-BE': ['fr', 'BE', ''], 'fr-CH': ['fr', 'CH', ''], 'fr-CA': ['fr', 'CA', '']}
version = u"0.4.6"
author = u"Olivier R."
name = u"Grammalecte"

import lightproof_handler_grammalecte as loopt

# end of sentence regex
reEOS = re.compile(u'([.?!:;…][ .?!… »”")]*|.$)')
# begin of paragraph regex
reBOP = re.compile(u"^\W*")
# end of paragraph regex
reEOP = re.compile(u"\W*$")

DEBUG = False

# loaded rules (check for Update mechanism of the editor)
try:
    langrule
except NameError:
    langrule = {}

# ignored rules
ignore = {}

# cache for morphogical analyses
analyses = {}
stems = {}
suggestions = {}

# assign Calc functions
calcfunc = None

# check settings (this is sloooooooow!)
#def option(lang, opt): 
#    try:
#        return loopt.get_option(lang.Language + "_" + lang.Country, opt)

def option (s):
    # replace the slow lightproof option
    return loopt.cache_options.get(s, False)


# filtering affix fields (ds, is, ts etc.)
def onlymorph(st):
    if st != None:
        st = re.sub(r"^.*(st:|po:)", r"\\1", st) # keep last word part
        st = re.sub(r"\\b(?=[dit][sp]:)","@", st) # and its affixes
        st = re.sub(r"(?<!@)\\b\w\w:\w+","", st).replace('@','').strip()
    return st

# if the pattern matches all analyses of the input word, 
# return the last matched substring
def _morph(rLoc, word, pattern, onlyaffix, strict, noword):
    global analyses
    if not word:
        return noword
    if word not in analyses or len(analyses[word]) == 0:
        x = spellchecker.spell(u"<?xml?><query type='analyze'><word>" + word + "</word></query>", rLoc, ())
        if not x:
            return None
        t = x.getAlternatives()
        if not t:
            t = [""]
        analyses[word] = t[0].split("</a>")[:-1]
    a = analyses[word]
    result = None
    p = re.compile(pattern)
    if strict :
        for i in a:
            if onlyaffix:
                i = onlymorph(i)
            result = p.search(i)
            if result:
                result = result.group(0)
            else:
                return None
        return result
    else:
        for i in a:
            if onlyaffix:
                i = onlymorph(i)
            result = p.search(i)
            if result:
                return result.group(0)
        return None

def morph(rLoc, word, pattern, strict=True, noword=False):
    return _morph(rLoc, word, pattern, False, strict, noword)

def affix(rLoc, word, pattern, strict=True, noword=False):
    return _morph(rLoc, word, pattern, True, strict, noword)

def spell(rLoc, word):
    if not word:
        return None
    return spellchecker.isValid(word, rLoc, ())

# get the tuple of the stem of the word or an empty array
def stem(rLoc, word):
    global stems
    if not word:
        return []
    if not word in stems or len(stems[word]) == 0:
        x = spellchecker.spell(u"<?xml?><query type='stem'><word>" + word + "</word></query>", rLoc, ())
        if not x:
            return []
        t = x.getAlternatives()
        if not t:
            t = []
        stems[word] = list(t)
    return stems[word]

# get the tuple of the morphological generation of a word or an empty array
def generate(rLoc, word, example):
    if not word:
        return []
    x = spellchecker.spell(u"<?xml?><query type='generate'><word>" + word + "</word><word>" + example + "</word></query>", rLoc, ())
    if not x:
        return []
    t = x.getAlternatives()
    if not t:
        return []
    return list(t)

# get suggestions
def suggest(rLoc, word):
    global suggestions
    if not word:
        return word
    if word not in suggestions:
        x = spellchecker.spell("_" + word, rLoc, ())
        if not x:
            return word
        t = x.getAlternatives()
        suggestions[word] = "|".join(t)
    return suggestions[word]

# get the nth word of the input string or None
def word(s, n):
    a = re.match(u"(?u)( +[-\w%]+){" + str(n-1) + u"} +([-\w%]+)", s)
    if not a:
        return ''
    return a.group(2)

# get the (-)nth word of the input string or None
def wordmin(s, n):
    a = re.search(u"(?u)([-\w%]+) +([-\w%]+ +){" + str(n-1) + u"}$", s)
    if not a:
        return ''
    return a.group(1)

def calc(funcname, par):
    global calcfunc
    global SMGR
    if calcfunc == None:
        calcfunc = SMGR.createInstance( "com.sun.star.sheet.FunctionAccess")
        if calcfunc == None:
                return None
    return calcfunc.callFunction(funcname, par)

import time

def parse (nDocId, rText, rLocale, nStartOfSentencePos, nSuggestedSentenceEndPos, rProperties):
    #start = time.clock()
    aErr = []
    # analyze by paragraph
    try:
        s, aErr = proofread( nDocId, rText, rLocale, nStartOfSentencePos, nSuggestedSentenceEndPos, rProperties, 0)
        if s:
            rText = s
    except:
        raise
    
    # analyze by sentence
    lPos = []
    nBOS = reBOP.match(rText).end() # nStartOfSentencePos
    for m in reEOS.finditer(rText):
        lPos.append((nBOS, m.end()))
        nBOS = m.end()
    for pos in lPos:
        nl = pos[1] - pos[0]
        if nl > 4 and nl < 1500:
            for i in range(1, get_npass()):
                try:
                    s, errs = proofread( nDocId, rText, rLocale, pos[0], pos[1], rProperties, i)
                    if s:
                        rText = rText[:pos[0]] + s + rText[pos[1]:]
                    aErr += errs
                except:
                    raise
    #end = time.clock()
    #print("elapsed time: %s\t <  %s" % (end-start, rText[:10].encode("ascii", errors="replace").decode("ascii", errors="replace")))
    return tuple(aErr)

def proofread( nDocId, TEXT, LOCALE, nStartOfSentencePos, nSuggestedSentenceEndPos, rProperties, nPass ):
    aErrs = []
    s = TEXT[nStartOfSentencePos:nSuggestedSentenceEndPos]
    bChange = False
    
    if nPass == 1:
        if u" " in s:
            s = s.replace(u" ", u' ') # nbsp
            bChange = True
        if u" " in s:
            s = s.replace(u" ", u' ') # snbsp
            bChange = True
        if u"@" in s:
            s = s.replace(u"@", u' ')
            bChange = True
        if u"'" in s:
            s = s.replace(u"'", u"’")
            bChange = True
        if u"‑" in s:
            s = s.replace(u"‑", u"-") # nobreakdash
            bChange = True
    if nPass == 3:
        if u"@" in s:
            s = s.replace(u"@", u' ')
            bChange = True

    for i in get_pp(nPass):
        # 0: regex,  1: replacement,  2: message,  3: condition,  4: ngroup,  5: numline/id (str), 6: option, 7: URL, 8: case sensitive?
        for m in i[0].finditer(s):
            try:
                if not i[3] or eval(i[3]):
                    ln = m.end(i[4]) - m.start(i[4])
                    if i[1] == "*":
                        rep = " " * ln
                    elif i[1] == ">" or i[1] == "_" or i[1] == "/":
                        rep = i[1] + " " * (ln-1)
                    elif i[1] == "@":
                        rep = "@" * ln
                    elif i[1][0:1] == "=":
                        if i[1][1:2] != "@":
                            rep = eval(i[1][1:])
                            rep = rep + " " * (ln-len(rep))
                        else:
                            rep = eval(i[1][2:])
                            rep = rep + "@" * (ln-len(rep))
                        if i[-1] and m.group(i[4])[0:1].isupper():
                            rep = rep.capitalize()
                    else:
                        rep = m.expand(i[1])
                        rep = rep + " " * (ln-len(rep))
                    s = s[0:m.start(i[4])] + rep + s[m.end(i[4]):]
                    bChange = True
                    if DEBUG:
                        print(u"> " + s.encode('ascii', 'replace').decode('ascii', 'replace') + " -- "+ m.group(i[4]).encode('ascii', 'replace').decode('ascii', 'replace') + "  #" + i[5])
            except Exception as e:
                raise Exception(str(e), i[5])
    
    for i in get_rule(nPass):
        # 0: regex,  1: replacement,  2: message,  3: condition,  4: ngroup,  5: numline/id (str), 6: option, 7: URL, 8: case sensitive?
        if i[0] and (i[6] == False or option(i[6])) and not i[5] in ignore:
            for m in i[0].finditer(s):
                try:
                    if not i[3] or eval(i[3]):
                        aErr = uno.createUnoStruct( "com.sun.star.linguistic2.SingleProofreadingError" )
                        aErr.nErrorStart        = nStartOfSentencePos + m.start(i[4]) # nStartOfSentencePos
                        aErr.nErrorLength       = m.end(i[4]) - m.start(i[4])
                        aErr.nErrorType         = PROOFREADING
                        aErr.aRuleIdentifier    = i[5]
                        iscap = (i[-1] and m.group(i[4])[0:1].isupper())
                        if i[1][0:1] == "=":
                            sugg = eval(i[1][1:])
                            aErr.aSuggestions = tuple(cap(sugg.split("|"), iscap)) if sugg  else ()
                        elif i[1] == "_":
                            aErr.aSuggestions = ()
                        else:
                            aErr.aSuggestions = tuple(cap(m.expand(i[1]).split("|"), iscap))
                        comment = i[2]
                        if comment[0:1] == "=":
                            comment = eval(comment[1:])
                        else:
                            comment = m.expand(comment)
                        aErr.aShortComment      = comment.split("|")[0].strip()   # in context menu
                        aErr.aFullComment       = comment.split("|")[-1].strip()  # in dialog
                        if DEBUG:
                            aErr.aShortComment += " -- " + i[5]
                        # URL
                        if i[7]:
                            p = PropertyValue()
                            p.Name = "FullCommentURL"
                            p.Value = i[7]
                            aErr.aProperties        = (p,)
                        else:
                            aErr.aProperties        = ()
                        aErrs = aErrs + [aErr]
                except Exception as e:
                    raise Exception(str(e), i[5])
    if bChange:
        return (s, tuple(aErrs))
    return (False, tuple(aErrs))

def cap(a, iscap):
    if iscap:
        for i in range(0, len(a)):
            a[i] = a[i].capitalize()
    return a

def compile_rules(dic):
    # compile regular expressions
    for i in dic:
        try:
            if i[0].startswith("(?iu)"):
                i += [True]
                i[0] = i[0].replace("(?iu)", "(?u)")
            else:
                i += [False]
            i[0] = re.compile(i[0])
        except:
            print("Lightproof: bad regular expression: ", traceback.format_exc())
            i[0] = None

def get_npass ():
    try:
        return len(langrule[pkg].dic)
    except:
        langrule[pkg] = __import__("lightproof_" + pkg)
        for i in range(len(langrule[pkg].dic)):
            compile_rules(langrule[pkg].dic[i])
            compile_rules(langrule[pkg].pp[i])
    return len(langrule[pkg].dic)

def get_rule (n):
    try:
        return langrule[pkg].dic[n]
    except:
        langrule[pkg] = __import__("lightproof_" + pkg)
        for i in range(len(langrule[pkg].dic)):
            compile_rules(langrule[pkg].dic[i])
            compile_rules(langrule[pkg].pp[i])
    return langrule[pkg].dic[n]

def get_pp (n):
    try:
        return langrule[pkg].pp[n]
    except:
        langrule[pkg] = __import__("lightproof_" + pkg)
        for i in range(len(langrule[pkg].dic)):
            compile_rules(langrule[pkg].dic[i])
            compile_rules(langrule[pkg].pp[i])
    return langrule[pkg].pp[n]

def get_path():
    return os.path.join(os.path.dirname(sys.modules[__name__].__file__), __name__ + ".py")


#### GRAMMALECTE

def seekMorphFromHunspell (rLoc, word):
    "retrieves morphologies list from Hunspell -> analyses"
    global analyses
    x = spellchecker.spell(u"<?xml?><query type='analyze'><word>" + word + "</word></query>", rLoc, ())
    if x:
        t = x.getAlternatives()
        analyses[word] = t[0].split("</a>")[:-1]  if t  else  []

def rewriteSubject (rLoc, s1, s2):
    # s1 is supposed to be prn/patr/npr
    if s2 == "lui":
        return "ils"
    if s2 == "moi":
        return "nous"
    if s2 == "toi":
        return "vous"
    if s2 == "nous":
        return "nous"
    if s2 == "vous":
        return "vous"
    if s2 == "eux":
        return "ils"
    if s2 == "elle" or s2 == "elles":
        # We don’t check if word exists in analyses, for it is assumed it has been done before
        if cr.mbNprMasNotFem(analyses.get(s1, "")):
            return "ils"
        # si épicène, indéterminable, mais OSEF, le féminin l’emporte
        return "elles"
    return s1 + " et " + s2

def word1 (s):
    "get next word (optimization)"
    m = cr.NextWord.match(s)
    if not m:
        return ''
    return m.group(1)

def wordmin1 (s):
    "get previous word (optimization)"
    m = cr.PrevWord.search(s)
    if not m:
        return ''
    return m.group(1)

def look (s, p, np=None):
    "seek pattern p in s (before/after/fulltext), if antipattern np not in s"
    if np and re.search(np, s):
        return False
    if re.search(p, s):
        return True
    return False

def lookchk1 (rLoc, s, p, pmg1, negpmg1=None):
    "returns True if s has pattern p and m.group(1) has pattern pmg1"
    m = re.search(p, s)
    if not m:
        return False
    try:
        word = m.group(1)
    except:
        #print("Missing group 1")
        return False
    if word not in analyses or len(analyses[word]) == 0:
        seekMorphFromHunspell(rLoc, word)
    a = analyses[word]
    if not a:
        return False
    if negpmg1:
        # check negative condition
        np = re.compile(negpmg1)
        for s in a:
            if np.search(s):
                return False
    # search pattern
    rp = re.compile(pmg1)
    for s in a:
        if rp.search(s):
            return True
    return False

def stemchk (rLoc, word, pattern, strict=True, noword=False):
    "check pattern on stems"
    global stems
    if not word:
        return noword
    if word not in stems or len(stems[word]) == 0:
        x = spellchecker.spell(u"<?xml?><query type='stem'><word>" + word + "</word></query>", rLoc, ())
        if not x:
            return False
        t = x.getAlternatives()
        if not t:
            t = ['']
        stems[word] = t
    a = stems[word]
    p = re.compile(pattern)
    if strict:
        for s in a:
            if not p.search(s):
                return False
        return True
    else:
        for s in a:
            if p.search(s):
                return True
        return False

def morphex (rLoc, word, pattern, negpattern, noword=False):
    "returns True if not negpattern in word and pattern in word"
    if not word:
        return noword
    if word not in analyses or len(analyses[word]) == 0:
        seekMorphFromHunspell(rLoc, word)
    a = analyses[word]
    if not a:
        return False
    # check negative condition
    np = re.compile(negpattern)
    for s in a:
        if np.search(s):
            return False
    # search pattern
    p = re.compile(pattern)
    for s in a:
        if p.search(s):
            return True
    return False

def apposition (word1, word2):
    "returns True if nom + nom = no agreement required"
    # We don’t check if word exists in analyses, for it is assumed it has been done before
    if cr.mbNomNotAdj(analyses.get(word2, "")) and cr.mbPpasNomNotAdj(analyses.get(word1, "")):
        return True
    return False

def isAmbiguousNAV (rLoc, word):
    "words which are nom|adj and verb are ambiguous (except être and avoir)"
    if word not in analyses or len(analyses[word]) == 0:
        seekMorphFromHunspell(rLoc, word)
    a = analyses[word]
    if not a:
        return False
    if not cr.mbNomAdj(a):
        return False
    if cr.mbVconj123(a) and not cr.mbMG(a):
        return True
    return False

def isAmbiguousAndWrong (sWord1, sWord2, sReqMorphNA, sReqMorphConj):
    "use it if sWord1 won’t be a verb; word2 is assumed to be True via isAmbiguousNAV"
    # We don’t check if word exists in analyses, for it is assumed it has been done before
    a2 = analyses.get(sWord2, [])
    if not a2:
        return False
    if cr.checkConjVerb(a2, sReqMorphConj):
        # verb word2 is ok
        return False
    a1 = analyses.get(sWord1, [])
    if not a1:
        return False
    if cr.checkAgreement(a1, a2, sReqMorphNA) and (cr.mbAdj(a2) or cr.mbAdj(a1)):
        return False
    return True

def isVeryAmbiguousAndWrong (sWord1, sWord2, sReqMorphNA, sReqMorphConj, bLastHopeCond):
    "use it if sWord1 can be also a verb; word2 is assumed to be True via isAmbiguousNAV"
    # We don’t check if word exists in analyses, for it is assumed it has been done before
    a2 = analyses.get(sWord2, [])
    if not a2:
        return False
    if cr.checkConjVerb(a2, sReqMorphConj):
        # verb word2 is ok
        return False
    a1 = analyses.get(sWord1, [])
    if not a1:
        return False
    if cr.checkAgreement(a1, a2, sReqMorphNA) and (cr.mbAdj(a2) or cr.mbAdjNb(a1)):
        return False
    # now, we know there no agreement, and conjugation is also wrong
    if cr.isNomAdj(a1):
        return True
    #if cr.isNomAdjVerb(a1): # considered True
    if bLastHopeCond:
        return True
    return False


#### Other

reGENDER = re.compile(":(mas|fem|epi)")
reNUMBER = re.compile(":(sg|pl|inv)")

def wordGender (rLoc, sWord):
    "returns gender of word ('mas', 'fem', 'epi' or empty string)."
    if sWord not in analyses or len(analyses[sWord]) == 0:
        seekMorphFromHunspell(rLoc, sWord)
    a = analyses[sWord]
    sGender = ""
    for sMorph in a:
        m = reGENDER.search(sMorph)
        if m:
            if not sGender:
                sGender = m.group(1)
            elif sGender != m.group(1):
                return "epi"
    return sGender

def wordNumber (rLoc, sWord):
    "returns number of word ('sg', 'pl', 'inv' or empty string)."
    if sWord not in analyses or len(analyses[sWord]) == 0:
        seekMorphFromHunspell(rLoc, sWord)
    a = analyses[sWord]
    sNumber = ""
    for sMorph in a:
        m = reNUMBER.search(sWord)
        if m:
            if not sNumber:
                sNumber = m.group(1)
            elif sNumber != m.group(1):
                return "inv"
    return sNumber


#### Syntagmes

reENDOFNG1 = re.compile(u" +(?:, +|)(?:n(’|e |ous )|l(?:’|e(?:urs? |) |a )|j(?:’|e )|m(?:’|e |a |on )|t(?:’|e |a |u )|s(?:’|e |a )|c(?:’|e(?:t |tte | ))|ç(?:a |’))")
reENDOFNG2 = re.compile(r" +(\w[\w-]+)")
reENDOFNG3 = re.compile(r" *, +(\w[\w-]+)")

def isEndOfNG (rLoc, s):
    if reENDOFNG1.match(s):
        return True
    m = reENDOFNG2.match(s)
    if m and morphex(rLoc, m.group(1), "po:(?:v|prep)", "po:(?:nom|adj|pp)"):
        return True
    m = reENDOFNG3.match(s)
    if m and not morph(rLoc, m.group(1), "po:(?:nom|adj)", False):
        return True
    return False


#### Suggestions mechanism

def suggVerb (rLoc, sFlex, sWho, funcSugg2=None):
    aSugg = set()
    for sStem in stem(rLoc, sFlex):
        tTags = conjfr.getTags(sStem)
        if tTags:
            # we get the tense
            aTense = set()
            for sMorph in analyses.get(sFlex, []): # we don’t check if word exists in analyses, for it is assumed it has been done before
                for m in re.finditer(sStem+" .* po:(i(?:pre|imp|psi|fut)|s(?:pre|imp)|cond)", sMorph):
                    # stem must be used in regex to prevent confusion between different verbs (i.e. sauras has 2 stems: savoir and saurer)
                    if m:
                        aTense.add(m.group(1))
            # TODO: define what tenses to use when sFlex is infinitive?
            for sTense in aTense:
                if sWho == "1isg" and not conjfr.hasConjWithTags(tTags, sTense, "1isg"):
                    sWho = "1sg"
                if conjfr.hasConjWithTags(tTags, sTense, sWho):
                    aSugg.add(conjfr.getConjWithTags(sStem, tTags, sTense, sWho))
    if funcSugg2:
        aSugg2 = funcSugg2(rLoc, sFlex)
        if aSugg2:
            aSugg.add(aSugg2)
    if aSugg:
        return u"|".join(aSugg)
    return ""

def suggPlur (rLoc, sFlex, sWordAgreement=None):
    "returns plural forms assuming sFlex is singular"
    if sWordAgreement:
        sGender = wordGender(rLoc, sWordAgreement)
        if sGender == "mas":
            return suggMasPlur(rLoc, sFlex)
        elif sGender == "fem":
            return suggFemPlur(rLoc, sFlex)
    aSugg = set()
    if "-" not in sFlex:
        if sFlex.endswith("l"):
            if sFlex.endswith("al") and len(sFlex) > 2 and spellchecker.isValid(sFlex[:-1]+"ux", rLoc, ()):
                aSugg.add(sFlex[:-1]+"ux")
            if sFlex.endswith("ail") and len(sFlex) > 3 and spellchecker.isValid(sFlex[:-2]+"ux", rLoc, ()):
                aSugg.add(sFlex[:-2]+"ux")
        if spellchecker.isValid(sFlex+"s", rLoc, ()):
            aSugg.add(sFlex+"s")
        if spellchecker.isValid(sFlex+"x", rLoc, ()):
            aSugg.add(sFlex+"x")
    if mfspfr.isMiscPlural(sFlex):
        aSugg.add(mfspfr.getMiscPlural(sFlex))
    if aSugg:
        return u"|".join(aSugg)
    return ""

def suggSing (rLoc, sFlex):
    "returns singular forms assuming sFlex is plural"
    if "-" in sFlex:
        return ""
    aSugg = set()
    if sFlex.endswith("ux"):
        if spellchecker.isValid(sFlex[:-2]+"l", rLoc, ()):
            aSugg.add(sFlex[:-2]+"l")
        if spellchecker.isValid(sFlex[:-2]+"il", rLoc, ()):
            aSugg.add(sFlex[:-2]+"il")
    if spellchecker.isValid(sFlex[:-1], rLoc, ()):
        aSugg.add(sFlex[:-1])
    if aSugg:
        return u"|".join(aSugg)
    return ""

def suggMasSing (rLoc, sFlex):
    "returns masculine singular forms"
    aSugg = set()
    for sMorph in analyses.get(sFlex, []): # we don’t check if word exists in analyses, for it is assumed it has been done before
        if not "po:v" in sMorph:
            # not a verb
            if "is:mas" in sMorph or "is:epi" in sMorph:
                aSugg.add(suggSing(rLoc, sFlex))
            else:
                sStem = cr.getLemmaOfMorph(sMorph)
                if mfspfr.isFemForm(sStem):
                    aSugg.add(mfspfr.getMasForm(sStem, False))
        else:
            # a verb
            sVerb = cr.getLemmaOfMorph(sMorph)
            if conjfr.hasConj(sVerb, "pp", "ppas1"):
                aSugg.add(conjfr.getConj(sVerb, "pp", "ppas1"))
    if aSugg:
        return u"|".join(aSugg)
    return ""

def suggMasPlur (rLoc, sFlex):
    "returns masculine plural forms"
    aSugg = set()
    for sMorph in analyses.get(sFlex, []): # we don’t check if word exists in analyses, for it is assumed it has been done before
        if not "po:v" in sMorph:
            # not a verb
            if "is:mas" in sMorph or "is:epi" in sMorph:
                aSugg.add(suggPlur(rLoc, sFlex))
            else:
                sStem = cr.getLemmaOfMorph(sMorph)
                if mfspfr.isFemForm(sStem):
                    aSugg.add(mfspfr.getMasForm(sStem, True))
        else:
            # a verb
            sVerb = cr.getLemmaOfMorph(sMorph)
            if conjfr.hasConj(sVerb, "pp", "ppas2"):
                aSugg.add(conjfr.getConj(sVerb, "pp", "ppas2"))
            elif conjfr.hasConj(sVerb, "pp", "ppas1"):
                aSugg.add(conjfr.getConj(sVerb, "pp", "ppas1"))
    if aSugg:
        return u"|".join(aSugg)
    return ""

def suggFemSing (rLoc, sFlex):
    "returns feminine singular forms"
    aSugg = set()
    for sMorph in analyses.get(sFlex, []): # we don’t check if word exists in analyses, for it is assumed it has been done before
        if not "po:v" in sMorph:
            # not a verb
            if "is:fem" in sMorph or "is:epi" in sMorph:
                aSugg.add(suggSing(rLoc, sFlex))
            else:
                sStem = cr.getLemmaOfMorph(sMorph)
                if mfspfr.isFemForm(sStem):
                    aSugg.add(sStem)
        else:
            # a verb
            sVerb = cr.getLemmaOfMorph(sMorph)
            if conjfr.hasConj(sVerb, "pp", "ppas3"):
                aSugg.add(conjfr.getConj(sVerb, "pp", "ppas3"))
    if aSugg:
        return u"|".join(aSugg)
    return ""

def suggFemPlur (rLoc, sFlex):
    "returns feminine plural forms"
    aSugg = set()
    for sMorph in analyses.get(sFlex, []): # we don’t check if word exists in analyses, for it is assumed it has been done before
        if not "po:v" in sMorph:
            # not a verb
            if "is:fem" in sMorph or "is:epi" in sMorph:
                aSugg.add(suggPlur(rLoc, sFlex))
            else:
                sStem = cr.getLemmaOfMorph(sMorph)
                if mfspfr.isFemForm(sStem):
                    aSugg.add(sStem+"s")
        else:
            # a verb
            sVerb = cr.getLemmaOfMorph(sMorph)
            if conjfr.hasConj(sVerb, "pp", "ppas4"):
                aSugg.add(conjfr.getConj(sVerb, "pp", "ppas4"))
    if aSugg:
        return u"|".join(aSugg)
    return ""

def switchGender (rLoc, sFlex, bPlur=None):
    # we don’t check if word exists in analyses, for it is assumed it has been done before
    aSugg = set()
    if bPlur == None:
        for sMorph in analyses.get(sFlex, []): 
            if "is:fem" in sMorph:
                if "is:sg" in sMorph:
                    aSugg.add(suggMasSing(rLoc, sFlex))
                elif "is:pl" in sMorph:
                    aSugg.add(suggMasPlur(rLoc, sFlex))
            elif "is:mas" in sMorph:
                if "is:sg" in sMorph:
                    aSugg.add(suggFemSing(rLoc, sFlex))
                elif "is:pl" in sMorph:
                    aSugg.add(suggFemPlur(rLoc, sFlex))
                else:
                    aSugg.add(suggFemSing(rLoc, sFlex))
                    aSugg.add(suggFemPlur(rLoc, sFlex))
    elif bPlur:
        for sMorph in analyses.get(sFlex, []):
            if "is:fem" in sMorph:
                aSugg.add(suggMasPlur(rLoc, sFlex))
            elif "is:mas" in sMorph:
                aSugg.add(suggFemPlur(rLoc, sFlex))
    else:
        for sMorph in analyses.get(sFlex, []):
            if "is:fem" in sMorph:
                aSugg.add(suggMasSing(rLoc, sFlex))
            elif "is:mas" in sMorph:
                aSugg.add(suggFemSing(rLoc, sFlex))

    if aSugg:
        return u"|".join(aSugg)
    return ""

def switchPlural (rLoc, sFlex):
     # we don’t check if word exists in analyses, for it is assumed it has been done before
    aSugg = set()
    for sMorph in analyses.get(sFlex, []):
        if "is:sg" in sMorph:
            aSugg.add(suggPlur(rLoc, sFlex))
        elif "is:pl" in sMorph:
            aSugg.add(suggSing(rLoc, sFlex))
    if aSugg:
        return u"|".join(aSugg)
    return ""


#### Check date validity

lDAY = ["lundi", "mardi", "mercredi", "jeudi", "vendredi", "samedi", "dimanche"]
dMONTH = { "janvier":1, u"février":2, "mars":3, "avril":4, "mai":5, "juin":6, "juillet":7, u"août":8, "aout":8, "septembre":9, "octobre":10, "novembre":11, u"décembre":12 }

import datetime

def checkDate (day, month, year):
    "to use if month is a number"
    try:
        return datetime.date(int(year), int(month), int(day))
    except ValueError:
        return False
    except:
        #traceback.print_exc()
        return True

def checkDateWithString (day, month, year):
    "to use if month is a noun"
    try:
        return datetime.date(int(year), dMONTH.get(month, ""), int(day))
    except ValueError:
        return False
    except:
        #traceback.print_exc()
        return True

def checkDay (weekday, day, month, year):
    "to use if month is a number"
    dt = checkDate(day, month, year)
    if dt and lDAY[dt.weekday()] != weekday.lower():
        return False
    return True
        
def checkDayWithString (weekday, day, month, year):
    "to use if month is a noun"
    dt = checkDate(day, dMONTH.get(month, ""), year)
    if dt and lDAY[dt.weekday()] != weekday.lower():
        return False
    return True

def getDay (day, month, year):
    "to use if month is a number"
    return lDAY[datetime.date(int(year), int(month), int(day)).weekday()]

def getDayWithString (day, month, year):
    "to use if month is a noun"
    return lDAY[datetime.date(int(year), dMONTH.get(month, ""), int(day)).weekday()]


# [code]


# Exceptions

aREPETPAREXCEP = frozenset(["quelquefois", "parfois", u"tant\xf4t", "souvent"])


def show (s):
    "Debug: \xe0 utiliser dans les conditions pour affichage"
    print("SHOW: " + s.encode("unicode-escape", "replace").decode("ascii", "replace"))
    return True

def formatNumber (s):
    nLen = len(s)
    if nLen == 10:
        sRes = s[0] + u"\xa0" + s[1:4] + u"\xa0" + s[4:7] + u"\xa0" + s[7:]                                  # nombre ordinaire
        if s.startswith("0"):
            sRes += u"|" + s[0:2] + u"\xa0" + s[2:4] + u"\xa0" + s[4:6] + u"\xa0" + s[6:8] + u"\xa0" + s[8:]    # t\xe9l\xe9phone fran\xe7ais
            if s[1] == "4" and (s[2]=="7" or s[2]=="8" or s[2]=="9"):
                sRes += u"|" + s[0:4] + u"\xa0" + s[4:6] + u"\xa0" + s[6:8] + u"\xa0" + s[8:]                # mobile belge
            sRes += u"|" + s[0:3] + u"\xa0" + s[3:6] + u"\xa0" + s[6:8] + u"\xa0" + s[8:]                    # t\xe9l\xe9phone suisse
        sRes += u"|" + s[0:4] + u"\xa0" + s[4:7] + "-" + s[7:]                                         # t\xe9l\xe9phone canadien ou am\xe9ricain
        return sRes 
    elif nLen == 9:
        sRes = s[0:3] + u"\xa0" + s[3:6] + u"\xa0" + s[6:]                                                # nombre ordinaire
        if s.startswith("0"):
            sRes += "|" + s[0:3] + u"\xa0" + s[3:5] + u"\xa0" + s[5:7] + u"\xa0" + s[7:9]                    # fixe belge 1
            sRes += "|" + s[0:2] + u"\xa0" + s[2:5] + u"\xa0" + s[5:7] + u"\xa0" + s[7:9]                    # fixe belge 2
        return sRes
    elif nLen < 4:
        return "_"
    sRes = ""
    nEnd = nLen
    while nEnd > 0:
        nStart = max(nEnd-3, 0)
        if not sRes:
            sRes = s[nStart:nEnd]
        else:
            sRes = s[nStart:nEnd] + u"\xa0" + sRes
        nEnd = nEnd - 3
    return sRes


